from ddt import ddt, data
from django.contrib.contenttypes.models import ContentType
from rest_framework import status, test

from waldur_core.cost_tracking import models as ct_models
from waldur_core.cost_tracking.tests import factories as ct_factories
from waldur_openstack.openstack_tenant import models as ot_models, cost_tracking as ot_cost_tracking
from waldur_openstack.openstack_tenant.tests import factories as ot_factories

from . import factories, fixtures


@ddt
class OpenStackTenantOptimizerTest(test.APITransactionTestCase):
    def setUp(self):
        self.fixture = fixtures.CostPlanningOpenStackPluginFixture()
        self.plan = self.fixture.deployment_plan
        self.url = factories.DeploymentPlanFactory.get_url(self.plan, action='evaluate')
        self.service = self.fixture.spl.service
        self.settings = self.service.settings
        self.flavor_params = [
            {'cores': 1, 'ram': 1 * 1024, 'name': 'flavor-1'},
            {'cores': 2, 'ram': 2 * 1024, 'name': 'flavor-2'},
            {'cores': 2, 'ram': 3 * 1024, 'name': 'flavor-3'},
            {'cores': 4, 'ram': 4 * 1024, 'name': 'flavor-4'},
        ]

        for p in self.flavor_params:
            ot_factories.FlavorFactory(settings=self.settings, **p)
            ct_models.DefaultPriceListItem.objects.update_or_create(
                resource_content_type=ContentType.objects.get_for_model(ot_models.Instance),
                item_type='flavor',
                key=p['name'])

        ct_factories.DefaultPriceListItemFactory(
            resource_content_type=ContentType.objects.get_for_model(ot_models.Volume),
            item_type=ot_cost_tracking.VolumeStrategy.Types.STORAGE,
            key=ot_cost_tracking.VolumeStrategy.Keys.STORAGE,
        )

    @data(
        {'variant': 'small', 'cores': 2, 'ram': 1 * 1024},
        {'variant': 'medium', 'cores': 2, 'ram': 4 * 1024},
    )
    def test_filter_flavors_if_exist_any_meet_plan_requirements(self, preset_param):
        response = self._get_response(preset_param)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        optimal_flavor = filter(lambda x: x['cores'] >= preset_param['cores'] and x['ram'] >= preset_param['ram'],
                                self.flavor_params)[0]['name']
        self.assertFalse(data[0]['error_message'])
        self.assertEqual(optimal_flavor, data[0]['optimized_presets'][0]['flavor']['name'])

    @data(
        {'variant': 'large', 'cores': 8, 'ram': 4 * 1024},
    )
    def test_filter_flavors_if_not_exist_any_meet_plan_requirements(self, preset_param):
        response = self._get_response(preset_param)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        data = response.json()
        self.assertTrue(data[0]['error_message'])
        self.assertTrue('It is too big' in data[0]['error_message'])

    def _get_response(self, preset_param):
        self.preset = factories.PresetFactory(category=self.fixture.category, **preset_param)
        factories.DeploymentPlanItemFactory(plan=self.plan, preset=self.preset)

        self.client.force_authenticate(self.fixture.staff)
        response = self.client.get(self.url)
        return response
